from enum import Enum

from hydbedscraper.drivers import hyderabad, ahmedabad


class City(Enum):
    AHMEDABAD = "ahmedabad"
    HYDERABAD = "hyderabad"

    def work(self):
        if self is City.AHMEDABAD:
            return ahmedabad.work()
        elif self is City.HYDERABAD:
            return hyderabad.work()
        else:
            raise NotImplementedError(self)


def get_city_from_str(city_name: str) -> City:
    try:
        return City(city_name)
    except ValueError as exc:
        raise ValueError(f"Supported cities: {[city for city in City]}") from exc



import logging

from hydbedscraper.city import get_city_from_str
from hydbedscraper.types import t_DataFrameDict


# TODO: setup proper logging
logging.getLogger().setLevel(logging.INFO)


def work(city_name: str) -> t_DataFrameDict:
    """
    :param city_name: lowercase name of city whose hospital-bed-availability info is required
    :return: string-to-DataFrame map
    """
    city = get_city_from_str(city_name)
    return city.work()



from typing import Callable, Dict


def clean_int(raw: str) -> int:
    newline_separations = raw.split("\n")
    for substr in newline_separations:
        try:
            int(substr)
        except ValueError:
            continue
        else:
            return int(substr)
    raise TypeError(f"could not clean {raw} for integer")


def clean_str(raw: str) -> str:
    return raw


dtype_to_cleaner_map: Dict[type, Callable] = {int: clean_int, str: clean_str}



import logging
import requests

from bs4 import BeautifulSoup
from urllib.parse import urljoin

from hydbedscraper.requesters.common import decode_streamed_string_response
from hydbedscraper.testdata.path import (
    test_government_hospital_data_html,
    test_private_hospital_data_html,
)
from hydbedscraper.types import t_BeautifulSoup

base_URL = "http://164.100.112.24/SpringMVC/"


def get_government_hospitals_page_soup(use_test: bool = True) -> t_BeautifulSoup:
    logging.info("getting government hospitals page..")
    if use_test:
        with open(test_government_hospital_data_html, "r") as fp:
            soup = BeautifulSoup(fp, "lxml")
    else:
        page = requests.post(
            urljoin(base_URL, "getHospital_Beds_Status_Citizen.htm"),
            data={
                "hospital": "G",
            },
            stream=True,
        )
        soup = BeautifulSoup(decode_streamed_string_response(page), "lxml")
    logging.info("..done")
    return soup


def get_private_hospitals_page_soup(use_test: bool = True) -> t_BeautifulSoup:
    logging.info("getting private hospitals page..")
    if use_test:
        with open(test_private_hospital_data_html, "r") as fp:
            soup = BeautifulSoup(fp, "lxml")
    else:
        page = requests.post(
            urljoin(base_URL, "getHospital_Beds_Status_Citizen.htm"),
            data={
                "hospital": "P",
            },
            stream=True,
        )
        soup = BeautifulSoup(decode_streamed_string_response(page), "lxml")
    logging.info("..done")
    return soup



from hydbedscraper.parsers.ahmedabad import parse_hospital_tables
from hydbedscraper.requesters.ahmedabad import get_hospital_tables
from hydbedscraper.writers.ahmedabad import to_dataframe_dict


def work():
    tables = get_hospital_tables()
    hospital_category_to_information_dict = parse_hospital_tables(tables)
    return to_dataframe_dict(hospital_category_to_information_dict)



import bs4
import camelot
import pandas
import requests

from typing import Dict

# bs4
t_BeautifulSoup = bs4.BeautifulSoup
t_Tag = bs4.Tag

# response
t_Response = requests.models.Response

# pandas
t_DataFrame = pandas.DataFrame
t_DataFrameDict = Dict[str, pandas.DataFrame]

# camelot
t_Table = camelot.core.Table
t_TableList = camelot.core.TableList



import camelot
import logging
import re

from urllib.parse import urljoin

import requests
from hydbedscraper.requesters.common import decode_streamed_string_response
from hydbedscraper.types import t_TableList

base_URL = "https://ahna.org.in/"


def get_hospital_tables() -> t_TableList:
    logging.info("getting hospital tables..")

    def _get_hospitals_pdf_url_suffix() -> str:
        hospitals_pdf_prefix = "AMC REQUISITIONED HOSPITAL STATUS"
        hospitals_pdf_suffix = "pdf"
        hospitals_pdf_regex_pattern = (
            fr"{hospitals_pdf_prefix}.*\.{hospitals_pdf_suffix}"
        )

        # get page
        page = requests.post(
            urljoin(base_URL, "covid19.html"),
            stream=True,
        )
        response_content = decode_streamed_string_response(page)

        # find pdf name in page content
        match_object = re.search(hospitals_pdf_regex_pattern, response_content)
        if match_object:
            pdf_name = match_object.group(0)
        else:
            raise ValueError("could not determine pdf name")

        # replace spaces
        pdf_url = pdf_name.replace(" ", "%20")
        return pdf_url

    # get pdf-url suffix
    pdf_url_suffix = _get_hospitals_pdf_url_suffix()

    # ask camelot to read pdf at url
    tables = camelot.read_pdf(
        urljoin(base_URL, pdf_url_suffix),
        pages="all",
        line_scale=40,  # necessary to detect smaller tables
    )

    logging.info("..done")
    return tables



import logging

from typing import Dict, List

from hydbedscraper.types import t_BeautifulSoup


def parse_government_hospitals(govt_soup: t_BeautifulSoup) -> Dict[str, List]:
    logging.info("parsing hospitals page..")
    col_id_of_district = 1
    key_colid_dtype = [
        ("hospital_name", 2, str),
        ("contact_number", 3, str),
        ("regular_bed_occupied", 5, int),
        ("regular_bed_vacant", 6, int),
        ("oxygen_bed_occupied", 8, int),
        ("oxygen_bed_vacant", 9, int),
        ("icu_bed_occupied", 11, int),
        ("icu_bed_vacant", 12, int),
        ("last_updated_date", 16, str),
        ("last_updated_time", 17, str),
    ]

    govt_dict: Dict[str, List] = {"district": []}
    for k, _, _ in key_colid_dtype:
        govt_dict[k] = []

    table = govt_soup.find("table")
    body = table.find("tbody")
    rows = body("tr")
    current_district: str = ""
    for row in rows:
        cells = row("td")
        # check if start of new district
        if len(cells) > 16:
            assert "rowspan" in cells[col_id_of_district].attrs
            current_district = cells[col_id_of_district].string
            offset = 0
        else:
            offset = -2
        govt_dict["district"].append(current_district)
        for k, kid, dtype in key_colid_dtype:
            govt_dict[k].append(dtype(cells[kid + offset].string.strip()))

    logging.info("..done")
    return govt_dict


def parse_private_hospitals(private_soup: t_BeautifulSoup) -> Dict[str, List]:
    logging.info("parsing hospitals page..")
    private_dict = parse_government_hospitals(private_soup)
    logging.info("..done")
    return private_dict



# contains absolute paths to testdata-files from project root

from pathlib import Path

_test_dir = Path(__file__).parent.absolute()

test_summary_html = _test_dir.joinpath("data_summary.html")
test_government_hospital_data_html = _test_dir.joinpath(
    "data_government_hospitals.html"
)
test_private_hospital_data_html = _test_dir.joinpath("data_private_hospitals.html")



import pandas as pd

from typing import Dict, List

from hydbedscraper.types import t_DataFrameDict

GOVT_DATAFRAME_KEY = "hyderabad_government_info"
PRIVATE_DATAFRAME_KEY = "hyderabad_private_info"


def to_dataframe_dict(
    govt_info: Dict[str, List], private_info: Dict[str, List]
) -> t_DataFrameDict:
    return {
        GOVT_DATAFRAME_KEY: pd.DataFrame.from_dict(govt_info),
        PRIVATE_DATAFRAME_KEY: pd.DataFrame.from_dict(private_info),
    }



from hydbedscraper.types import t_Response


def decode_streamed_string_response(response: t_Response) -> str:
    response_content = ""
    for chunk in response.iter_content(8192, decode_unicode=True):
        response_content += chunk
    return response_content



from hydbedscraper.parsers.hyderabad import (
    parse_government_hospitals,
    parse_private_hospitals,
)
from hydbedscraper.requesters.hyderabad import (
    get_government_hospitals_page_soup,
    get_private_hospitals_page_soup,
)
from hydbedscraper.writers.hyderabad import to_dataframe_dict


def work():
    govt_soup = get_government_hospitals_page_soup(use_test=False)
    private_soup = get_private_hospitals_page_soup(use_test=False)

    govt_info = parse_government_hospitals(govt_soup)
    private_info = parse_private_hospitals(private_soup)

    return to_dataframe_dict(govt_info, private_info)



import pandas as pd
import re

from collections import defaultdict
from typing import Dict, Tuple, Optional

from hydbedscraper.parsers._cleaners import dtype_to_cleaner_map
from hydbedscraper.types import t_TableList, t_Table

col_id_of_serial_number = 0
invalid_category_strings = ["Status of vacancy of Private beds", "O = Occupied"]
unknown_category_tag = "UNKNOWN_HOSPITAL_CATEGORY"
dchc_category_tag = "DCHC"
dchc_first_cell_label = "TOTAL DCHC"
dchc_index_offset = -2

key_colid_dtype = [
    ("hospital_name", 2, str),
    ("isolation_occupied", 3, int),
    ("isolation_vacant", 4, int),
    ("hdu_occupied", 5, int),
    ("hdu_vacant", 6, int),
    ("icu_no_ventilator_occupied", 7, int),
    ("icu_no_ventilator_vacant", 8, int),
    ("icu_ventilator_occupied", 9, int),
    ("icu_ventilator_vacant", 10, int),
]
date_key = "date"
time_key = "time"


def _get_date_and_time(table_list: t_TableList) -> Tuple[str, str]:
    # ASSMUPTION: Date and time are stored in first column of some row
    for table in table_list:
        date_time_regex = r".*Date: *(.*) *Time: *(.*)"
        for cell in table.df[0]:
            regex_result = re.search(date_time_regex, cell)
            if regex_result:
                date_str = regex_result.group(1)
                time_str = regex_result.group(2)
                return date_str, time_str
    return "??/??/????", "??:?? ??"


def _find_category_name(
    table: t_Table, search_start_row: int, current_hospital_category: Optional[str]
) -> str:
    # go backwards up the rows
    for row_index in range(search_start_row, -1, -1):
        row = table.df.iloc[row_index, :]
        # ASSUMPTION: row with category information contains only one occupied cell
        all_columns_except_first_column_are_empty = pd.DataFrame.all(row[1:] == "")
        if all_columns_except_first_column_are_empty:
            category_candidate = row[0]
            # filter out invalid candidates
            valid = True
            for invalid_category_string in invalid_category_strings:
                if invalid_category_string in category_candidate:
                    valid = False
                    break
            if valid:
                return category_candidate
            else:
                continue
        else:
            # continue trying earlier rows
            continue
    # if not found, check if current category exists
    if current_hospital_category is not None:
        return current_hospital_category
    else:
        return unknown_category_tag


def _get_hospital_category_information_for_table(
    table: t_Table, current_hospital_category: Optional[str]
) -> Dict[str, Tuple[int, int]]:
    category_to_start_end_index_tuple_map = dict()
    active_category = None
    start_index = None
    for row_index, val in enumerate(table.df[col_id_of_serial_number]):
        try:
            # see if row has serial number
            # ASSUMPTION: serial number can be converted to integer
            int(val)
        except ValueError:
            # cannot convert to integer
            if start_index is not None:
                # we have come to the end of a category
                # store end index and prepare for next block
                assert start_index is not None
                assert active_category is not None
                end_index = row_index - 1
                category_to_start_end_index_tuple_map[active_category] = (
                    start_index,
                    end_index,
                )
                start_index = None
                active_category = None
            else:
                # no category block has begun yet, and no serial number row has been found yet
                # continue looping through rows
                continue
        else:
            # the row begins with a serial number - therefore it has data
            if row_index == 0:
                # the first row already starts with data
                # assume category is active from earlier table
                assert start_index is None
                assert current_hospital_category is not None
                active_category = current_hospital_category
                start_index = row_index
            elif row_index == table.df.shape[0] - 1:
                # end of page, table continues into the next page
                assert start_index is not None
                assert active_category is not None
                end_index = row_index
                category_to_start_end_index_tuple_map[active_category] = (
                    start_index,
                    end_index,
                )
                start_index = None
                active_category = None
            elif active_category is None:
                # we just found a new category!
                active_category = _find_category_name(
                    table, row_index - 1, current_hospital_category
                )
                start_index = row_index
            else:
                # nothing to do, we are still going through a block
                continue
    return category_to_start_end_index_tuple_map


def parse_hospital_tables(hospital_tables: t_TableList) -> Dict[str, Dict[str, list]]:
    hospital_category_to_information_dict: Dict[str, Dict[str, list]] = defaultdict(
        lambda: defaultdict(list)
    )
    # read date and time
    date_str, time_str = _get_date_and_time(hospital_tables)

    # current hospital-category
    current_hospital_category = None

    for table in hospital_tables:
        # get map from hospital-category to start and end row-index of that category
        category_to_start_end_index_tuple_map = (
            _get_hospital_category_information_for_table(
                table, current_hospital_category
            )
        )

        # go over each hospital-category
        for category, (
            start_index,
            end_index,
        ) in category_to_start_end_index_tuple_map.items():
            # go over each row for that category
            for row_index in range(start_index, end_index + 1):
                # go over each information-cell
                for key, col_id, dtype in key_colid_dtype:
                    # add information
                    raw = table.df.iloc[row_index, col_id]
                    cleaned = dtype_to_cleaner_map[dtype](raw)
                    hospital_category_to_information_dict[category][key].append(cleaned)
                # add date and time information
                hospital_category_to_information_dict[category][date_key].append(
                    date_str
                )
                hospital_category_to_information_dict[category][time_key].append(
                    time_str
                )
            # store current hospital category
            current_hospital_category = category

        if len(category_to_start_end_index_tuple_map.keys()) == 0:
            # ASSUMPTION: Dedicated COVID Health Center (DCHC) table has no serial number information
            # Detect it with the label of its summary row
            for row_index in range(table.shape[0]):
                first_cell = table.df[0][row_index]
                if isinstance(first_cell, str):
                    if dchc_first_cell_label in first_cell:
                        for key, col_id, dtype in key_colid_dtype:
                            # add information
                            raw = table.df.iloc[row_index, col_id + dchc_index_offset]
                            cleaned = dtype_to_cleaner_map[dtype](raw)
                            hospital_category_to_information_dict[dchc_category_tag][
                                key
                            ].append(cleaned)
                        # add date and time information
                        hospital_category_to_information_dict[dchc_category_tag][
                            date_key
                        ].append(date_str)
                        hospital_category_to_information_dict[dchc_category_tag][
                            time_key
                        ].append(time_str)

    return hospital_category_to_information_dict



import pandas as pd

from typing import Dict


def to_dataframe_dict(
    hospital_category_to_information_dict: Dict[str, Dict[str, list]]
):
    return {
        hospital_category: pd.DataFrame.from_dict(information)
        for hospital_category, information in hospital_category_to_information_dict.items()
    }
